#ifndef __DEVICE_CONTROL__
#define __DEVICE_CONTROL__

#include <sys/types.h>
#include <linux/kdev_t.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SM_DEVCTL_PERM 0x89b2
#define SM_DEVCTL_VIDPID_PERM 0x89b3
#define NICNAME_MAX 128

enum dectl_perm {
    DEVCTL_ENABLE = 1,      /* 启用 */
    DEVCTL_DISABLE = 2,     /* 禁用 */
    DEVCTL_RO = 5           /* 只读 */
};

enum interface_type {
    INF_SERIAL = 1,      /* 串口 */
    INF_PARALLEL,        /* 并口 */
    INF_1394,            /* 1394 接口 */
    INF_PCMCIA,          /* PCMCIA 接口*/
    INF_TYPEC,          /* Type-C 接口 */
    INF_HDMI,            /* HDMI 接口 */ 
    INF_DISPLAYPORT,    /* DisplayPort 接口 */
    INF_MAX
};

enum device_type {
    DEV_BLUETOOTH = 1,         /* 蓝牙 */
    DEV_INFRARED,              /* 红外 */
    DEV_MODEM,                 /* modem 设备*/
    DEV_STORAGE_DRIVE,         /* 存储驱动器(SD、TF 卡) */
    DEV_SCSI_RAID,             /* SCSI 及 RAID 控制器 */
    DEV_WINCE,                 /* wince 设备(安卓手机、pad 等) */
    DEV_FLOPPY_DRIVE,          /* 软驱 */
    DEV_IMAGE,                 /* 图像设备(包括摄像头、
                   数码相机、扫描仪) */
    DEV_MULTI_SERIAL_CARD,     /* 多串口卡*/
    DEV_ETHERNET,              /* 以太网 */
    DEV_WIRELESS,              /* 无线 */
    DEV_PRINTER,               /* 打印机*/
    DEV_PRINTER_LOCAL,         /* 本地打印机 */
    DEV_PRINTER_NET,           /* 网络打印机 */
    DEV_CDROM,                 /* 刻录*/
    DEV_VOLUMESHADOW,          /* 卷影设备 */
    DEV_TAPE,                  /* 磁带机 */
    DEV_MAX
};

enum usb_subtype {
    USB_CLASS_PER_INTERFACE = 1,     /* 00h,保留分类 */
    USB_CLASS_AUDIO,                 /* 01h,音频设备 */
    USB_CLASS_COMM,                  /* 02h,通信及通信类控制设备 */
    USB_CLASS_HID,                     /* 03h,人体工学设备 */
    USB_CLASS_PHYSICAL,                /* 05h,物理设备 */
    USB_CLASS_STILL_IMAGE,             /* 06h,图像 */
    USB_CLASS_PRINTER,                 /* 07h,打印设备 */
    USB_CLASS_MASS_STORAGE,            /* 08h,大容量存储 */
    USB_CLASS_HUB,                     /* 09h,集线器 */
    USB_CLASS_CDC_DATA,                /* 0Ah,通信类数据设备 */
    USB_CLASS_CSCID,                   /* 0Bh,智能卡 */
    USB_CLASS_CONTENT_SEC,             /* 0Dh,内容安全 */
    USB_CLASS_VIDEO,                   /* 0Eh,视频 */
    USB_CLASS_WIRELESS_CONTROLLER,     /* E0h,无线控制器 */
    USB_CLASS_MISC,                    /* EFh,其他设备 */
    USB_CLASS_APP_SPEC,                /* FEh,应用程序 */
    USB_CLASS_VENDOR_SPEC,             /* FFh,厂家自定义 */
    USB_CLASS_PERSONAL_HEALTH,         /* 0F,个人健康 */
    USB_CLASS_DIAGNOSTIC,              /* DC，诊断设备 */
    USB_MAX,
    USB_CLASS_MAX = USB_MAX
};

struct devctl_special_device_policy {
    int usb_class;         /* 指定 usb 设备的子 class 类别 */
    unsigned int pid;     /* 指定设备的 pid */
    unsigned int vid;    /* 指定设备的 vid */
    int perm;             /* 指定设备的权限 */
};

struct devctl_netcards {
    char netcards[NICNAME_MAX];
};

struct devctl_policy_info {
    int inf_type;         /* 见上面的接口枚举 */
    int dev_type;         /* 见上面的设备枚举 */
    int usb_type;         /* 见上面的 usb 设备枚举 */
    int perm;             /* 设备(接口)权限 */
};

/*
 * 功能说明: 检测系统环境是否为安全套件环境
 * 参数说明: 无
 * 返回值:   0：成功, 非0：失败
 */
int devctl_aqtj_kernel_decide();

/*
 * 功能说明: 初始化安全套件环境默认设置
 * 参数说明: 无
 * 返回值:   0：成功, 非0：失败
 */
int devctl_default_init_policy();

/*
 * 功能说明: 添加接口的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_add_inf_policy(struct devctl_policy_info *recv);

/*
 * 功能说明: 删除接口的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_remove_inf_policy(struct devctl_policy_info *recv);

/*
 * 功能说明: 添加设备的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_add_device_policy(struct devctl_policy_info *recv);

/*
 * 功能说明: 删除设备的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_remove_device_policy(struct devctl_policy_info *recv);

/*
 * 功能说明: 添加USB设备的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_add_usb_policy(struct devctl_policy_info *recv);

/*
 * 功能说明: 删除USB设备的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_remove_usb_policy(struct devctl_policy_info *recv);

/*
 * 功能说明: 添加指定设备的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_add_special_device_policy(struct devctl_special_device_policy *recv);

/*
 * 功能说明: 删除指定设备的管控策略
 * 参数说明: recv: 策略信息的结构体
 * 返回值:   0：成功, 非0：失败
 */
int devctl_remove_special_device_policy(struct devctl_special_device_policy *recv);

/*
 * 功能说明: 获取当前外设管控所有的管控策略
 * 参数说明: count: 策略个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_policy_info *devctl_get_all_policies(int *count);

/*
 * 功能说明: 获取接口的管控策略
 * 参数说明: count: 策略个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_policy_info *devctl_get_inf_policies(int *count);

/*
 * 功能说明: 获取设备的管控策略
 * 参数说明: count: 策略个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_policy_info *devctl_get_device_policies(int *count);

/*
 * 功能说明: 获取USB设备的管控策略
 * 参数说明: count: 策略个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_policy_info *devctl_get_usb_device_policies(int *count);

/*
 * 功能说明: 获取白名单指定设备的管控策略
 * 参数说明: count: 策略个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_special_device_policy *devctl_get_special_device_policies(int *count);

/*
 * 功能说明: 获取所有指定设备的管控策略
 * 参数说明: count: 策略个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_special_device_policy *devctl_get_all_special_device_policies(int *count);

/*
 * 功能说明: 将指定网卡名加入黑名单（禁止）
 * 参数说明: netcard_name: 网卡名称
 * 返回值:   0 成功，-1 失败,-2 黑名单中已存在
 */
int devctl_sm_netcard_add(const char *netcard_name);

/*
 * 功能说明: 将指定网卡名从黑名单中删除（启用）
 * 参数说明: netcard_name: 网卡名称
 * 返回值:   0 成功，-1 失败，-2黑名单中不存在
 */
int devctl_sm_netcard_del(const char *netcard_name);

/*
 * 功能说明: 获取黑名单中网卡个数
 * 参数说明: count: 网卡个数
 * 返回值:   Null:失败 , 非Null:成功
 */
struct devctl_netcards *devctl_get_all_netcard_blacklist(int *count);

#ifdef __cplusplus
}
#endif

#endif
